Index: lib/matplotlib/legend.py
===================================================================
--- lib/matplotlib/legend.py	(revision 8396)
+++ lib/matplotlib/legend.py	(working copy)
@@ -253,7 +253,15 @@
 
         # introduce y-offset for handles of the scatter plot
         if scatteryoffsets is None:
-            self._scatteryoffsets = np.array([3./8., 4./8., 2.5/8.])
+            if scatterpoints<2:
+                self._scatteryoffsets = np.array([.5])
+            elif scatterpoints==2:
+                self._scatteryoffsets = np.array([.5,.5])
+            elif scatterpoints==3:
+                self._scatteryoffsets = np.array([3./8., 4./8., 2.5/8.])
+            else:
+                self._scatteryoffsets = np.ones(scatterpoints)*(3./8.)
+                self._scatteryoffsets[1::2] = 4./8.
         else:
             self._scatteryoffsets = np.asarray(scatteryoffsets)
         reps =  int(self.numpoints / len(self._scatteryoffsets)) + 1
@@ -446,6 +454,7 @@
         """
 
         fontsize = self._fontsize
+        markerscale = self.markerscale
 
         # legend_box is a HPacker, horizontally packed with
         # columns. Each column is a VPacker, vertically packed with
@@ -519,6 +528,9 @@
                 legline_marker.set_clip_box(None)
                 legline_marker.set_clip_path(None)
                 legline_marker.set_linestyle('None')
+                if markerscale !=1:
+                    newsz = legline_marker.get_markersize()*markerscale
+                    legline_marker.set_markersize(newsz)
                 # we don't want to add this to the return list because
                 # the texts and handles are assumed to be in one-to-one
                 # correpondence.
@@ -551,23 +563,20 @@
 
             elif isinstance(handle, RegularPolyCollection):
 
-                #ydata = self._scatteryoffsets
                 ydata = height*self._scatteryoffsets
 
                 size_max, size_min = max(handle.get_sizes()),\
                                      min(handle.get_sizes())
-                # we may need to scale these sizes by "markerscale"
-                # attribute. But other handle types does not seem
-                # to care about this attribute and it is currently ignored.
+                
                 if self.scatterpoints < 4:
-                    sizes = [.5*(size_max+size_min), size_max,
-                             size_min]
+                    sizes = np.array([.5*(size_max+size_min), size_max,
+                             size_min])
                 else:
                     sizes = (size_max-size_min)*np.linspace(0,1,self.scatterpoints)+size_min
 
                 p = type(handle)(handle.get_numsides(),
                                  rotation=handle.get_rotation(),
-                                 sizes=sizes,
+                                 sizes=sizes*markerscale,
                                  offsets=zip(xdata_marker,ydata),
                                  transOffset=self.get_transform(),
                                  )
@@ -588,12 +597,12 @@
                 # attribute. But other handle types does not seem
                 # to care about this attribute and it is currently ignored.
                 if self.scatterpoints < 4:
-                    sizes = [.5*(size_max+size_min), size_max,
-                             size_min]
+                    sizes = np.array([.5*(size_max+size_min), size_max,
+                             size_min])
                 else:
                     sizes = (size_max-size_min)*np.linspace(0,1,self.scatterpoints)+size_min
 
-                p = type(handle)(sizes,
+                p = type(handle)(sizes*markerscale,
                                  offsets=zip(xdata_marker,ydata),
                                  transOffset=self.get_transform(),
                                  )
